@@ -10,7 +10,7 @@
 # You should have received a copy of the GNU General Public License
 # along with this program. If not, see <http://www.gnu.org/licenses/>.
 
-from gi.repository import Gdk, GLib, Gio
+from gi.repository import Gdk, GLib
 
 import unicodedata
 import string
@@ -23,41 +23,6 @@ from eolie.logger import Logger
 from eolie.define import App, ArtSize, LoadingType
 
 
-def set_proxy_from_gnome():
-    """
-        Set proxy settings from GNOME
-    """
-    try:
-        proxy = Gio.Settings.new("org.gnome.system.proxy")
-        mode = proxy.get_value("mode").get_string()
-        if mode == "manual":
-            no_http_proxy = True
-            http = Gio.Settings.new("org.gnome.system.proxy.http")
-            https = Gio.Settings.new("org.gnome.system.proxy.https")
-            h = http.get_value("host").get_string()
-            p = http.get_value("port").get_int32()
-            hs = https.get_value("host").get_string()
-            ps = https.get_value("port").get_int32()
-            if h != "" and p != 0:
-                no_http_proxy = False
-                GLib.setenv("http_proxy", "http://%s:%s" % (h, p), True)
-            if hs != "" and ps != 0:
-                no_http_proxy = False
-                GLib.setenv("https_proxy", "http://%s:%s" % (hs, ps), True)
-            if no_http_proxy:
-                socks = Gio.Settings.new("org.gnome.system.proxy.socks")
-                h = socks.get_value("host").get_string()
-                p = socks.get_value("port").get_int32()
-                # Set socks proxy
-                if h != "" and p != 0:
-                    import socket
-                    import socks
-                    socks.set_default_proxy(socks.SOCKS4, h, p)
-                    socket.socket = socks.socksocket
-    except Exception as e:
-        Logger.error("set_proxy_from_gnome(): %s", e)
-
-
 def name_from_profile_id(id):
     """
         Get profile name from id
@@ -103,7 +68,7 @@ def wanted_loading_type(index):
     """
     if index == 0:
         return LoadingType.FOREGROUND
-    elif index < 10:
+    elif index == 1:
         return LoadingType.BACKGROUND
     else:
         return LoadingType.OFFLOAD
@@ -227,13 +192,13 @@ def get_current_monitor_model(window):
 
 
 def noaccents(string):
-        """
-            Return string without accents
-            @param string as str
-            @return str
-        """
-        nfkd_form = unicodedata.normalize('NFKD', string)
-        return u"".join([c for c in nfkd_form if not unicodedata.combining(c)])
+    """
+        Return string without accents
+        @param string as str
+        @return str
+    """
+    nfkd_form = unicodedata.normalize('NFKD', string)
+    return u"".join([c for c in nfkd_form if not unicodedata.combining(c)])
 
 
 def get_ftp_cmd():
