@@ -59,7 +59,7 @@ class WebViewNavigation:
         if App().phishing.is_phishing(uri):
             self._show_phishing_error(uri)
             return
-        self._error = False
+        self.discard_error()
         # If not an URI, start a search
         parsed = urlparse(uri)
         is_uri = parsed.scheme in ["about", "http",
@@ -92,9 +92,48 @@ class WebViewNavigation:
                 self.reset_bad_tls()
                 self.__insecure_content_detected = False
             self.stop_loading()
-            self.__update_settings_for_uri(uri)
+            self.update_settings_for_uri(uri)
             GLib.idle_add(WebKit2.WebView.load_uri, self, uri)
 
+    def update_settings_for_uri(self, uri):
+        """
+            Update internal settings for URI
+            @param uri as str
+        """
+        parsed = urlparse(uri)
+        http_scheme = parsed.scheme in ["http", "https"]
+        App().history.set_page_state(uri)
+        self.__switch_profile(uri)
+        self.__update_bookmark_metadata(uri)
+        self.content_manager.remove_all_style_sheets()
+        # Can't find a way to block content for ephemeral views
+        if App().settings.get_value("adblock") and\
+                not App().adblock_exceptions.find_parsed(parsed) and\
+                http_scheme and\
+                self.content_manager is not None:
+            self.content_manager.add_style_sheet(
+                App().default_style_sheet)
+            rules = App().adblock.get_css_rules(uri)
+            user_style_sheet = WebKit2.UserStyleSheet(
+                rules,
+                WebKit2.UserContentInjectedFrames.ALL_FRAMES,
+                WebKit2.UserStyleLevel.USER,
+                None,
+                None)
+            self.content_manager.add_style_sheet(user_style_sheet)
+        user_agent = App().websettings.get_user_agent(uri)
+        settings = self.get_settings()
+        if user_agent:
+            settings.set_user_agent(user_agent)
+        else:
+            settings.set_user_agent_with_application_details("Eolie",
+                                                             None)
+        # Setup image blocker
+        block_image = http_scheme and\
+            App().settings.get_value("imageblock") and\
+            not App().image_exceptions.find_parsed(parsed)
+        self.set_setting("auto-load-images", not block_image)
+
     @property
     def profile(self):
         """
@@ -112,8 +151,25 @@ class WebViewNavigation:
             @param webview as WebView
             @param event as WebKit2.LoadEvent
         """
-        parsed = urlparse(self._uri)
-        if event == WebKit2.LoadEvent.COMMITTED:
+        parsed = urlparse(self.uri)
+        if event == WebKit2.LoadEvent.STARTED:
+            self.emit("uri-changed", self.uri)
+        elif event == WebKit2.LoadEvent.REDIRECTED:
+            # Block ads
+            if App().settings.get_value("adblock") and\
+                    webview.__related_view is not None and\
+                    parsed.scheme in ["http", "https"] and\
+                    not App().adblock_exceptions.find_parsed(parsed):
+                if App().adblock.is_netloc_blocked(parsed.netloc) or\
+                        App().adblock.is_uri_blocked(self.uri,
+                                                     parsed.netloc):
+                    Logger.debug("WebView::__wait_for_uri(): blocking %s",
+                                 self.uri)
+                    webview.stop_loading()
+                    self._window.container.close_view(self.view)
+                    return
+        elif event == WebKit2.LoadEvent.COMMITTED:
+            self.emit("uri-changed", self.uri)
             http_scheme = parsed.scheme in ["http", "https"]
             self.update_zoom_level()
             # Setup eolie internal adblocker
@@ -133,11 +189,9 @@ class WebViewNavigation:
                             self.run_javascript(js, None, None)
                             break
         elif event == WebKit2.LoadEvent.FINISHED:
-            self.update_spell_checking(self._uri)
+            self.update_spell_checking(self.uri)
             self.run_javascript_from_gresource(
-                                  "/org/gnome/Eolie/Extensions.js", None, None)
-            if parsed.scheme in ["http", "https"]:
-                self.set_snapshot()
+                "/org/gnome/Eolie/Extensions.js", None, None)
             if App().show_tls:
                 try:
                     from OpenSSL import crypto
@@ -169,46 +223,6 @@ class WebViewNavigation:
 #######################
 # PRIVATE             #
 #######################
-    def __update_settings_for_uri(self, uri):
-        """
-            Update internal settings for URI
-            @param uri as str
-        """
-        parsed = urlparse(uri)
-        http_scheme = parsed.scheme in ["http", "https"]
-        App().history.set_page_state(uri)
-        self.__switch_profile(uri)
-        self.__update_bookmark_metadata(uri)
-        self.__hw_acceleration_policy(parsed.netloc)
-        self.content_manager.remove_all_style_sheets()
-        # Can't find a way to block content for ephemeral views
-        if App().settings.get_value("adblock") and\
-                not App().adblock_exceptions.find_parsed(parsed) and\
-                http_scheme and\
-                self.content_manager is not None:
-            self.content_manager.add_style_sheet(
-                                             App().default_style_sheet)
-            rules = App().adblock.get_css_rules(uri)
-            user_style_sheet = WebKit2.UserStyleSheet(
-                         rules,
-                         WebKit2.UserContentInjectedFrames.ALL_FRAMES,
-                         WebKit2.UserStyleLevel.USER,
-                         None,
-                         None)
-            self.content_manager.add_style_sheet(user_style_sheet)
-        user_agent = App().websettings.get_user_agent(uri)
-        settings = self.get_settings()
-        if user_agent:
-            settings.set_user_agent(user_agent)
-        else:
-            settings.set_user_agent_with_application_details("Eolie",
-                                                             None)
-        # Setup image blocker
-        block_image = http_scheme and\
-            App().settings.get_value("imageblock") and\
-            not App().image_exceptions.find_parsed(parsed)
-        self.set_setting("auto-load-images", not block_image)
-
     def __update_bookmark_metadata(self, uri):
         """
             Update bookmark access time/popularity
@@ -218,18 +232,6 @@ class WebViewNavigation:
             App().bookmarks.set_access_time(uri, round(time(), 2))
             App().bookmarks.set_more_popular(uri)
 
-    def __hw_acceleration_policy(self, netloc):
-        """
-            Disable hw acceleration for blacklist
-            @param netloc as str
-        """
-        blacklist = ["plus.google.com"]
-        if netloc in blacklist:
-            policy = WebKit2.HardwareAccelerationPolicy.NEVER
-        else:
-            policy = WebKit2.HardwareAccelerationPolicy.ON_DEMAND
-        self.get_settings().set_hardware_acceleration_policy(policy)
-
     def __switch_profile(self, uri):
         """
             Handle cookies manager
@@ -243,8 +245,8 @@ class WebViewNavigation:
             cookie_manager = self.get_context().get_cookie_manager()
             path = COOKIES_PATH % (EOLIE_DATA_PATH, profile)
             cookie_manager.set_persistent_storage(
-                                    path,
-                                    WebKit2.CookiePersistentStorage.SQLITE)
+                path,
+                WebKit2.CookiePersistentStorage.SQLITE)
 
     def __on_run_as_modal(self, webview):
         Logger.info("WebView::__on_run_as_modal(): TODO")
@@ -275,18 +277,17 @@ class WebViewNavigation:
 
     def __on_uri_changed(self, webview, param):
         """
-            Stop running background task and update favicon
+            Handle JS updates
             @param webview as WebKit2.WebView
             @param param as GObject.ParamSpec
         """
-        uri = webview.get_property(param.name)
-        self.stop_snapshot()
-        # JS bookmark (Bookmarklet)
-        if not uri.startswith("javascript:"):
-            self.emit("uri-changed", uri)
         # Js update, force favicon caching for current uri
         if not self.is_loading():
-            self.set_favicon()
+            uri = webview.get_property(param.name)
+            # JS bookmark (Bookmarklet)
+            if not uri.startswith("javascript:") and not self.error:
+                self.emit("uri-changed", uri)
+                self.set_current_favicon()
 
     def __on_title_changed(self, webview, param):
         """
@@ -322,6 +323,7 @@ class WebViewNavigation:
             parsed = urlparse(uri)
             if mime_type in self.__MIMES:
                 decision.use()
+                return False
             elif parsed.scheme == "file":
                 f = Gio.File.new_for_uri(uri)
                 info = f.query_info("standard::type",
@@ -336,13 +338,16 @@ class WebViewNavigation:
                         Logger.error("""WebViewNavigation::
                                         __on_decide_policy(): %s""", e)
                     decision.ignore()
+                    return True
                 else:
                     decision.use()
+                    return False
             elif self.can_show_mime_type(mime_type):
                 decision.use()
+                return False
             else:
                 decision.download()
-            return False
+                return True
 
         navigation_action = decision.get_navigation_action()
         navigation_uri = navigation_action.get_request().get_uri()
@@ -368,37 +373,40 @@ class WebViewNavigation:
                 self.new_page(navigation_uri, LoadingType.FOREGROUND)
                 decision.ignore()
                 return True
+            elif App().phishing.is_phishing(navigation_uri):
+                self._show_phishing_error(navigation_uri)
+                decision.ignore()
+                return True
             else:
+                self.discard_error()
                 decision.use()
-                self._error = False
                 return False
         elif mouse_button == 1:
             if decision_type == WebKit2.PolicyDecisionType.NEW_WINDOW_ACTION:
-                if self._window.modifiers & Gdk.ModifierType.SHIFT_MASK:
+                if self._window.modifiers == Gdk.KEY_Shift_L:
                     loading_type = LoadingType.POPOVER
                 else:
                     loading_type = LoadingType.FOREGROUND
                 self.new_page(navigation_uri, loading_type)
                 decision.ignore()
                 return True
-            elif self._window.modifiers & Gdk.ModifierType.CONTROL_MASK:
+            elif self._window.modifiers == Gdk.KEY_Control_L:
                 self.new_page(navigation_uri, LoadingType.BACKGROUND)
                 decision.ignore()
                 return True
-            elif self._window.modifiers & Gdk.ModifierType.SHIFT_MASK:
+            elif self._window.modifiers == Gdk.KEY_Shift_L:
                 self.new_page(navigation_uri, LoadingType.POPOVER)
                 decision.ignore()
                 return True
+            elif App().phishing.is_phishing(navigation_uri):
+                self._show_phishing_error(navigation_uri)
+                decision.ignore()
+                return True
             else:
-                self.__update_settings_for_uri(navigation_uri)
-                if App().phishing.is_phishing(navigation_uri):
-                    self._show_phishing_error(navigation_uri)
-                    decision.ignore()
-                    return True
-                else:
-                    self._error = False
-                    decision.use()
-                    return False
+                self.update_settings_for_uri(navigation_uri)
+                self.discard_error()
+                decision.use()
+                return False
         else:
             self.new_page(navigation_uri, LoadingType.BACKGROUND)
             decision.ignore()
